QueryScratch

Struct QueryScratch 

pub struct QueryScratch<SS: StemStrategy, O, const K: usize> { /* private fields */ }
Reusable traversal scratch storage for query builder scratch APIs.

The concrete stack representation is selected by the stem strategy and is intentionally hidden so it can keep changing during the v6 alpha cycle.

Implementations

Source

impl<SS: StemStrategy, O, const K: usize> QueryScratch<SS, O, K>

pub fn new() -> Self

Creates empty query traversal scratch storage.