pub struct QueryScratch<SS: StemStrategy, O, const K: usize> { /* private fields */ }Expand description
Reusable traversal scratch storage for query builder scratch APIs.
The concrete stack representation is selected by the stem strategy and is intentionally hidden so it can keep changing during the v6 alpha cycle.
Implementations
Sourceimpl<SS: StemStrategy, O, const K: usize> QueryScratch<SS, O, K>
impl<SS: StemStrategy, O, const K: usize> QueryScratch<SS, O, K>